@@ -1,12 +1,13 @@
 import asyncio
 import os
 import shutil
+import sys
 from typing import Any, AsyncIterator, Dict, Optional, TypeVar, Union
 
 from . import azure_auth
 from .boost import BoostExecutor, EagerAsyncIterator, consume
 from .globals import config
-from .listing import DirEntry, scantree
+from .listing import DirEntry, glob_scandir, scantree
 from .path import (
     AzurePath,
     BasePath,
@@ -14,7 +15,6 @@ from .path import (
     GooglePath,
     LocalPath,
     exists,
-    isfile,
     pathdispatch,
     url_format,
 )
@@ -169,26 +169,39 @@ async def _azure_cloudcopyfile(src: AzurePath, dst: AzurePath, overwrite: bool =
         copy_id = resp.headers["x-ms-copy-id"]
         copy_status = resp.headers["x-ms-copy-status"]
 
-    # wait for potentially async copy operation to finish
-    # https://docs.microsoft.com/en-us/rest/api/storageservices/get-blob
-    # pending, success, aborted, failed
-    sleep = exponential_sleep_generator(
-        initial=config.backoff_initial,
-        maximum=config.backoff_max,
-        jitter_fraction=config.backoff_jitter_fraction,
-    )
-    while copy_status == "pending":
-        await asyncio.sleep(next(sleep))
-        request = await azurify_request(
-            Request(
-                method="GET",
-                url=dst.format_url("https://{account}.blob.core.windows.net/{container}/{blob}"),
-            )
+    try:
+        # wait for potentially async copy operation to finish
+        # https://docs.microsoft.com/en-us/rest/api/storageservices/get-blob
+        # pending, success, aborted, failed
+        sleep = exponential_sleep_generator(
+            initial=config.backoff_initial,
+            maximum=config.backoff_max,
+            jitter_fraction=config.backoff_jitter_fraction,
         )
-        async with request.execute() as resp:
-            if resp.headers["x-ms-copy-id"] != copy_id:
-                raise RuntimeError("Copy id mismatch")
-            copy_status = resp.headers["x-ms-copy-status"]
+        while copy_status == "pending":
+            await asyncio.sleep(next(sleep))
+            request = await azurify_request(
+                Request(
+                    method="GET",
+                    url=dst.format_url(
+                        "https://{account}.blob.core.windows.net/{container}/{blob}"
+                    ),
+                )
+            )
+            async with request.execute() as resp:
+                if resp.headers["x-ms-copy-id"] != copy_id:
+                    raise RuntimeError("Copy id mismatch")
+                copy_status = resp.headers["x-ms-copy-status"]
+    except BaseException:
+        print(
+            "[boostedblob] Asynchronous cloud copy operation is still pending. You can cancel "
+            f"the copy manually with `az storage blob copy cancel --copy-id {copy_id} "
+            f"--destination-blob {dst.blob} --destination-container {dst.container} "
+            f"--account-name {dst.account}`. Note if successfully cancelled, this will produce "
+            "an empty blob at the destination.",
+            file=sys.stderr,
+        )
+        raise
     if copy_status != "success":
         raise RuntimeError(f"Invalid copy status: '{copy_status}'")
 
@@ -255,15 +268,10 @@ async def copytree_iterator(
     if isinstance(src, LocalPath):
         src = LocalPath(os.path.abspath(src.path))
 
-    dirpath = src.ensure_directory_like()
-    subpath_exists = False
-
     async def copy_wrapper(entry: DirEntry) -> BasePath:
-        nonlocal subpath_exists
-        subpath_exists = True
-
         assert isinstance(dst, BasePath)
         if entry.is_dir:
+            # Filter out directory marker files
             return entry.path
         size = entry.stat.size if entry.stat else None
         await copyfile(
@@ -271,16 +279,9 @@ async def copytree_iterator(
         )
         return entry.path
 
-    async for path in executor.map_unordered(copy_wrapper, EagerAsyncIterator(scantree(dirpath))):
+    async for path in executor.map_unordered(copy_wrapper, EagerAsyncIterator(scantree(src))):
         yield path
 
-    # If we find nothing, then run some checks so we throw the appropriate error.
-    # Doing this means we avoid extra requests in the happy path.
-    if not subpath_exists:
-        if not await isfile(src):
-            raise FileNotFoundError(src)
-        raise NotADirectoryError(src)
-
 
 @pathdispatch
 async def copytree(
@@ -314,4 +315,38 @@ async def _local_copytree(
         await consume(copytree_iterator(src, dst, executor))
         return
     assert isinstance(dst, LocalPath)
-    shutil.copytree(src, dst)
+    kwargs = {} if sys.version_info < (3, 8) else {"dirs_exist_ok": True}
+    shutil.copytree(src, dst, **kwargs)  # type: ignore[arg-type]
+
+
+# ==============================
+# copyglob
+# ==============================
+
+
+async def copyglob_iterator(
+    src: BasePath, dst: Union[BasePath, str], executor: BoostExecutor
+) -> AsyncIterator[BasePath]:
+    """Copies files matching the glob ``src`` into the directory ``dst``.
+
+    :param src: The glob to match against.
+    :param dst: The destination directory.
+    :param executor: An executor.
+
+    """
+    if isinstance(dst, str):
+        dst = BasePath.from_str(dst)
+    dst = dst.ensure_directory_like()
+
+    async def copy_wrapper(entry: DirEntry) -> Optional[BasePath]:
+        assert isinstance(dst, BasePath)
+        if entry.is_dir:
+            # Skip directories (or marker files for directories)
+            return None
+        size = entry.stat.size if entry.stat else None
+        await copyfile(entry.path, dst / entry.path.name, executor, size=size, overwrite=True)
+        return entry.path
+
+    async for path in executor.map_unordered(copy_wrapper, EagerAsyncIterator(glob_scandir(src))):
+        if path:
+            yield path
