@@ -3,8 +3,8 @@ import shutil
 from typing import AsyncIterator, Union
 
 from .boost import BoostExecutor, EagerAsyncIterator, consume
-from .listing import listtree
-from .path import AzurePath, BasePath, CloudPath, GooglePath, LocalPath, isfile, pathdispatch
+from .listing import glob_scandir, listtree
+from .path import AzurePath, BasePath, CloudPath, GooglePath, LocalPath, isdir, isfile, pathdispatch
 from .request import Request, azurify_request, googlify_request
 
 # ==============================
@@ -13,7 +13,7 @@ from .request import Request, azurify_request, googlify_request
 
 
 @pathdispatch
-async def remove(path: Union[BasePath, str]) -> None:
+async def remove(path: Union[BasePath, str]) -> BasePath:
     """Delete the file ``path``.
 
     :param path: The path to delete.
@@ -23,7 +23,7 @@ async def remove(path: Union[BasePath, str]) -> None:
 
 
 @remove.register  # type: ignore
-async def _azure_remove(path: AzurePath) -> None:
+async def _azure_remove(path: AzurePath) -> AzurePath:
     request = await azurify_request(
         Request(
             method="DELETE",
@@ -32,11 +32,18 @@ async def _azure_remove(path: AzurePath) -> None:
             failure_exceptions={404: FileNotFoundError(path)},
         )
     )
-    await request.execute_reponseless()
+    try:
+        await request.execute_reponseless()
+    except FileNotFoundError:
+        # Note that this continues to allow the deletion of directory marker files
+        if await isdir(path):
+            raise IsADirectoryError(path) from None
+        raise
+    return path
 
 
 @remove.register  # type: ignore
-async def _google_remove(path: GooglePath) -> None:
+async def _google_remove(path: GooglePath) -> GooglePath:
     request = await googlify_request(
         Request(
             method="DELETE",
@@ -45,12 +52,40 @@ async def _google_remove(path: GooglePath) -> None:
             failure_exceptions={404: FileNotFoundError(path)},
         )
     )
-    await request.execute_reponseless()
+    try:
+        await request.execute_reponseless()
+    except FileNotFoundError:
+        # Note that this continues to allow the deletion of directory marker files
+        if await isdir(path):
+            raise IsADirectoryError(path) from None
+        raise
+    return path
 
 
 @remove.register  # type: ignore
-async def _local_remove(path: LocalPath) -> None:
+async def _local_remove(path: LocalPath) -> LocalPath:
     os.remove(path)
+    return path
+
+
+# ==============================
+# glob_remove
+# ==============================
+
+
+async def glob_remove(
+    path: Union[BasePath, str], executor: BoostExecutor
+) -> AsyncIterator[BasePath]:
+    """Delete all files that match the glob ``path``.
+
+    :param path: The glob to match against.
+    :param executor: An executor.
+
+    """
+    async for subpath in executor.map_unordered(
+        lambda x: remove(x.path), EagerAsyncIterator(glob_scandir(path))
+    ):
+        yield subpath
 
 
 # ==============================
@@ -58,7 +93,7 @@ async def _local_remove(path: LocalPath) -> None:
 # ==============================
 
 
-async def rmtree_iterator(path: CloudPath, executor: BoostExecutor) -> AsyncIterator[CloudPath]:
+async def rmtree_iterator(path: BasePath, executor: BoostExecutor) -> AsyncIterator[BasePath]:
     """Delete the directory ``path``.
 
     Yields the deleted paths as they are deleted.
@@ -69,13 +104,8 @@ async def rmtree_iterator(path: CloudPath, executor: BoostExecutor) -> AsyncIter
     """
     dirpath = path.ensure_directory_like()
 
-    async def remove_wrapper(entry: CloudPath) -> CloudPath:
-        await remove(entry)
-        return entry
-
-    it: EagerAsyncIterator[CloudPath] = EagerAsyncIterator(listtree(dirpath))  # type: ignore
     try:
-        async for subpath in executor.map_unordered(remove_wrapper, it):
+        async for subpath in executor.map_unordered(remove, EagerAsyncIterator(listtree(dirpath))):
             yield subpath
     except FileNotFoundError:
         if await isfile(path):
