@@ -3,10 +3,22 @@ from __future__ import annotations
 import asyncio
 import contextlib
 import functools
+import os
 import sys
 import time
 from dataclasses import dataclass
-from typing import Any, AsyncIterator, Awaitable, Callable, Dict, Iterator, Optional, Tuple, TypeVar
+from typing import (
+    Any,
+    AsyncIterator,
+    Awaitable,
+    Callable,
+    Dict,
+    Generic,
+    Iterator,
+    Optional,
+    Tuple,
+    TypeVar,
+)
 
 import aiohttp
 
@@ -15,28 +27,26 @@ from . import azure_auth, google_auth
 MB = 2 ** 20
 
 
+T = TypeVar("T")
 F = TypeVar("F", bound=Callable[..., Any])
 
 
-class TokenManager:
+class TokenManager(Generic[T]):
     """Automatically refresh a token when it expires."""
 
-    def __init__(self, get_token_fn: Callable[[str], Awaitable[Tuple[Any, float]]]) -> None:
+    def __init__(self, get_token_fn: Callable[[T], Awaitable[Tuple[Any, float]]]) -> None:
         self._get_token_fn = get_token_fn
-        self._tokens: Dict[str, Any] = {}
-        self._expiration: Optional[float] = None
+        self._tokens: Dict[T, Any] = {}
+        self._expirations: Dict[T, float] = {}
 
-    async def get_token(self, key: str) -> Any:
+    async def get_token(self, key: T) -> Any:
         now = time.time()
-        if (
-            self._expiration is None
-            or (now + config.token_early_expiration_seconds) > self._expiration
-        ):
-            if key in self._tokens:
-                del self._tokens[key]
+        expiration = self._expirations.get(key)
+        if expiration is None or (now + config.token_early_expiration_seconds) > expiration:
+            self._tokens[key], self._expirations[key] = await self._get_token_fn(key)
+            assert self._expirations[key] is not None
 
-        if key not in self._tokens:
-            self._tokens[key], self._expiration = await self._get_token_fn(key)
+        assert key in self._tokens
         return self._tokens[key]
 
 
@@ -44,19 +54,23 @@ class TokenManager:
 class Config:
     session: Optional[aiohttp.ClientSession] = None
 
-    chunk_size: int = 32 * MB
+    debug_mode: bool = bool(os.environ.get("BBB_DEBUG"))
 
-    connect_timeout: float = 10.0
-    read_timeout: float = 30.0
+    chunk_size: int = 16 * MB
+
+    connect_timeout: float = 30.0
+    read_timeout: float = 300.0
     backoff_initial: float = 0.1
     backoff_max: float = 60.0
     backoff_jitter_fraction: float = 0.9
-    retry_limit: int = 100
+    retry_limit: int = 15
 
     token_early_expiration_seconds: int = 300
-    azure_access_token_manager: TokenManager = TokenManager(azure_auth.get_access_token)
-    azure_sas_token_manager: TokenManager = TokenManager(azure_auth.get_sas_token)
-    google_access_token_manager: TokenManager = TokenManager(google_auth.get_access_token)
+    azure_access_token_manager = TokenManager[Tuple[str, Optional[str]]](
+        azure_auth.get_access_token
+    )
+    azure_sas_token_manager = TokenManager[Tuple[str, Optional[str]]](azure_auth.get_sas_token)
+    google_access_token_manager = TokenManager[str](google_auth.get_access_token)
 
 
 config: Config = Config()
@@ -121,10 +135,15 @@ def set_event_loop_exception_handler() -> None:
         # Our main interest here is minimising the other various errors and tracebacks that
         # drown out the politely formatted errors from cli.py when things go wrong
         if "exception was never retrieved" in message:
-            print(
-                f"ERROR (while closing down): {type(exception).__name__}: {exception}",
-                file=sys.stderr,
-            )
+            from .request import MissingSession
+
+            # While closing down, we remove the global session, causing other requests to fail. This
+            # just causes noise, so ignore if that's the exception.
+            if not isinstance(exception, MissingSession):
+                print(
+                    f"ERROR (while closing down): {type(exception).__name__}: {exception}",
+                    file=sys.stderr,
+                )
         else:
             print(
                 f"ERROR (from event loop): {type(exception).__name__}: {message}", file=sys.stderr
