@@ -1,13 +1,25 @@
 from __future__ import annotations
 
 import configparser
+import datetime
 import os
+import re
 from typing import Any, AsyncIterator, Iterator, Mapping, NamedTuple, Optional, Union
 
 from . import google_auth
 from .path import AzurePath, BasePath, CloudPath, GooglePath, LocalPath, Stat, isfile, pathdispatch
 from .request import Request, azure_page_iterator, google_page_iterator
 
+
+def format_size(num: float, suffix: str = "B") -> str:
+    for unit in ["", "Ki", "Mi", "Gi", "Ti", "Pi", "Ei", "Zi"]:
+        if abs(num) < 1024.0:
+            unit += suffix
+            return f"{num:.1f} {unit:<3}"
+        num /= 1024.0
+    return f"{num:.1f} Yi{suffix}"
+
+
 # ==============================
 # DirEntry
 # ==============================
@@ -29,6 +41,15 @@ class DirEntry(NamedTuple):
         assert not path.is_directory_like()
         return DirEntry(path=path, is_dir=False, is_file=True, stat=stat)
 
+    def format(self, human_readable: bool = False) -> str:
+        size = (
+            (format_size(self.stat.size) if human_readable else self.stat.size) if self.stat else ""
+        )
+        mtime = (
+            datetime.datetime.fromtimestamp(int(self.stat.mtime)).isoformat() if self.stat else ""
+        )
+        return f"{size:>12}  {mtime:19}  {self.path}"
+
 
 # ==============================
 # list_blobs
@@ -36,25 +57,33 @@ class DirEntry(NamedTuple):
 
 
 @pathdispatch
-def list_blobs(path: Union[CloudPath, str], delimiter: Optional[str]) -> AsyncIterator[DirEntry]:
+def list_blobs(
+    path: Union[CloudPath, str], delimiter: Optional[str], allow_prefix: bool = False
+) -> AsyncIterator[DirEntry]:
     """List all blobs whose prefix matches ``path``.
 
     :param path: The prefix we want to list.
     :param delimiter: Group blobs matching the prefix up to a delimiter. This allows us to emulate
         directories on blob storage.
+    :param allow_prefix: Whether to allow prefixes that do not end with a delimiter (or "/").
 
     """
     raise ValueError(f"Unsupported path: {path}")
 
 
 @list_blobs.register  # type: ignore
-async def _azure_list_blobs(path: AzurePath, delimiter: Optional[str]) -> AsyncIterator[DirEntry]:
+async def _azure_list_blobs(
+    path: AzurePath, delimiter: Optional[str], allow_prefix: bool = False
+) -> AsyncIterator[DirEntry]:
     prefix = path
-    if delimiter:
-        assert not prefix.blob or prefix.blob.endswith(delimiter)
+    if not allow_prefix:
+        _delimiter = delimiter or "/"
+        assert not prefix.blob or prefix.blob.endswith(_delimiter)
     if not path.container:
+        if path.blob:
+            raise ValueError("Path is missing container; do you have an extra slash?")
         if delimiter != "/":
-            raise ValueError("Cannot list blobs in storage account")
+            raise ValueError("Cannot list blobs in storage account; must specify container")
         async for entry in _azure_list_containers(prefix.account):
             yield entry
         return
@@ -76,10 +105,13 @@ async def _azure_list_blobs(path: AzurePath, delimiter: Optional[str]) -> AsyncI
 
 
 @list_blobs.register  # type: ignore
-async def _google_list_blobs(path: GooglePath, delimiter: Optional[str]) -> AsyncIterator[DirEntry]:
+async def _google_list_blobs(
+    path: GooglePath, delimiter: Optional[str], allow_prefix: bool = False
+) -> AsyncIterator[DirEntry]:
     prefix = path
-    if delimiter:
-        assert not prefix.blob or prefix.blob.endswith(delimiter)
+    if not allow_prefix:
+        _delimiter = delimiter or "/"
+        assert not prefix.blob or prefix.blob.endswith(_delimiter)
     if not path.bucket:
         if delimiter != "/":
             raise ValueError("Cannot list blobs across buckets")
@@ -143,11 +175,11 @@ async def _local_scandir(path: LocalPath) -> AsyncIterator[DirEntry]:
         # entry.path is absolute since we're passing an absolute path to os.scandir
         entry_path = LocalPath(entry.path)
         if entry.is_dir():
-            yield DirEntry(path=entry_path, is_dir=True, is_file=False, stat=None)
-        else:
             yield DirEntry(
-                path=entry_path, is_dir=False, is_file=True, stat=Stat.from_local(entry.stat())
+                path=entry_path.ensure_directory_like(), is_dir=True, is_file=False, stat=None
             )
+        else:
+            yield DirEntry(path=entry_path, is_dir=False, is_file=True, stat=_os_safe_stat(entry))
 
 
 # ==============================
@@ -225,7 +257,7 @@ async def _local_scantree(path: LocalPath) -> AsyncIterator[DirEntry]:
                     path=LocalPath(entry.path),
                     is_dir=False,
                     is_file=True,
-                    stat=Stat.from_local(entry.stat()),
+                    stat=_os_safe_stat(entry),
                 )
 
     for entry in inner(path_absolute):
@@ -261,6 +293,92 @@ async def _local_listtree(path: LocalPath) -> AsyncIterator[LocalPath]:
             yield LocalPath(base) / name
 
 
+# ==============================
+# glob_scandir
+# ==============================
+
+
+@pathdispatch
+def glob_scandir(path: Union[BasePath, str]) -> AsyncIterator[DirEntry]:
+    """Iterate over a glob in a directory.
+
+    :param path: The glob to match against.
+
+    """
+    raise ValueError(f"Unsupported path: {path}")
+
+
+@glob_scandir.register  # type: ignore
+async def _azure_glob_scandir(path: AzurePath) -> AsyncIterator[DirEntry]:
+    if "*" in path.account:
+        raise ValueError("Cannot use wildcard in storage account")
+    if "*" in path.container:
+        if path.blob:
+            raise ValueError("Currently only supports wildcards inside the filename")
+        pattern = _glob_to_regex(path.container)
+        async for entry in _azure_list_containers(path.account):
+            assert isinstance(entry.path, type(path))
+            if re.match(pattern, entry.path.container):
+                yield entry
+        return
+    if "*" in os.path.dirname(path.blob):
+        raise ValueError("Currently only supports wildcards inside the filename")
+    if "*" not in path.name:
+        path = path / "*"
+    pattern = _glob_to_regex(path.name)
+
+    blob_prefix = path.blob.split("*", maxsplit=1)[0]
+    prefix = AzurePath(account=path.account, container=path.container, blob=blob_prefix)
+    async for entry in list_blobs(prefix, delimiter="/", allow_prefix=True):
+        assert isinstance(entry.path, type(path))
+        if entry.path == path.parent.ensure_directory_like():
+            # skip directory file marker of the parent, if present
+            continue
+        if re.match(pattern, entry.path.name):
+            yield entry
+
+
+@glob_scandir.register  # type: ignore
+async def _google_glob_scandir(path: GooglePath) -> AsyncIterator[DirEntry]:
+    if "*" in path.bucket:
+        if path.blob:
+            raise ValueError("Currently only supports wildcards inside the filename")
+        pattern = _glob_to_regex(path.bucket)
+        async for entry in _google_list_buckets():
+            assert isinstance(entry.path, type(path))
+            if re.match(pattern, entry.path.bucket):
+                yield entry
+        return
+    if "*" in os.path.dirname(path.blob):
+        raise ValueError("Currently only supports wildcards inside the filename")
+    if "*" not in path.name:
+        path = path / "*"
+    pattern = _glob_to_regex(path.name)
+
+    blob_prefix = path.blob.split("*", maxsplit=1)[0]
+    prefix = GooglePath(bucket=path.bucket, blob=blob_prefix)
+    async for entry in list_blobs(prefix, delimiter="/", allow_prefix=True):
+        assert isinstance(entry.path, type(path))
+        if entry.path == path.parent.ensure_directory_like():
+            # skip directory file marker of the parent, if present
+            continue
+        if re.match(pattern, entry.path.name):
+            yield entry
+
+
+@glob_scandir.register  # type: ignore
+async def _local_glob_scandir(path: LocalPath) -> AsyncIterator[DirEntry]:
+    if "*" in os.path.dirname(path):
+        raise ValueError("Currently only supports wildcards inside the filename")
+    if "*" not in path.name:
+        path = path / "*"
+    pattern = _glob_to_regex(path.name)
+
+    async for entry in scandir(path.parent):
+        if re.match(pattern, entry.path.name):
+            yield entry
+
+
 # ==============================
 # helpers
 # ==============================
@@ -360,8 +478,28 @@ async def _google_list_buckets(project: Optional[str] = None) -> AsyncIterator[D
         assert isinstance(buckets, list)
         for bucket in buckets:
             yield DirEntry(
-                path=GooglePath(bucket["name"], ""),
-                is_dir=True,
-                is_file=False,
-                stat=None,
+                path=GooglePath(bucket["name"], ""), is_dir=True, is_file=False, stat=None
             )
+
+
+def _glob_to_regex(pattern: str) -> str:
+    tokens = (token for token in re.split("([*]+)", pattern) if token != "")
+    regexp = ""
+    for token in tokens:
+        if token == "*":
+            regexp += r"[^/]*"
+        elif token == "**":
+            regexp += r".*"
+        else:
+            regexp += re.escape(token)
+    return regexp + r"/?$"
+
+
+def _os_safe_stat(entry: os.DirEntry[Any]) -> Optional[Stat]:
+    # stat can fail and we don't want that to affect listing operations
+    # In particular, we ran into the equivalent of the following raising FileNotFoundError:
+    # next(x for x in os.scandir("/") if x.name == ".VolumeIcon.icns").stat()
+    try:
+        return Stat.from_local(entry.stat())
+    except Exception:
+        return None
