@@ -9,9 +9,7 @@ import urllib.parse
 from dataclasses import dataclass
 from typing import Any, Callable, Mapping, NamedTuple, Optional, TypeVar, Union
 
-import xmltodict
-
-from .request import Request, azurify_request, googlify_request
+from .request import Request, azure_page_iterator, azurify_request, googlify_request
 
 T = TypeVar("T")
 
@@ -173,6 +171,9 @@ class AzurePath(CloudPath):
     def __str__(self) -> str:
         return f"https://{self.account}.blob.core.windows.net/{self.container}/{self.blob}"
 
+    def to_az_str(self) -> str:
+        return f"az://{self.account}/{self.container}/{self.blob}"
+
     def format_url(self, template: str) -> str:
         return url_format(template, account=self.account, container=self.container, blob=self.blob)
 
@@ -375,7 +376,8 @@ async def _azure_isdir(path: AzurePath) -> bool:
             prefix: str = path.blob
             if not prefix.endswith("/"):
                 prefix += "/"
-            request = await azurify_request(
+
+            it = azure_page_iterator(
                 Request(
                     method="GET",
                     url=path.format_url("https://{account}.blob.core.windows.net/{container}"),
@@ -389,12 +391,10 @@ async def _azure_isdir(path: AzurePath) -> bool:
                     failure_exceptions={404: FileNotFoundError()},
                 )
             )
-            async with request.execute() as resp:
-                body = await resp.read()
-                result = xmltodict.parse(body)["EnumerationResults"]
-                return result["Blobs"] is not None and (
-                    "BlobPrefix" in result["Blobs"] or "Blob" in result["Blobs"]
-                )
+            async for result in it:
+                if result["Blobs"] is not None:
+                    return "BlobPrefix" in result["Blobs"] or "Blob" in result["Blobs"]
+            return False
         else:
             request = await azurify_request(
                 Request(
@@ -404,8 +404,8 @@ async def _azure_isdir(path: AzurePath) -> bool:
                     failure_exceptions={404: FileNotFoundError()},
                 )
             )
-            async with request.execute() as resp:
-                return True
+            await request.execute_reponseless()
+            return True
     except FileNotFoundError:
         # execute might raise FileNotFoundError if storage account doesn't exist
         return False
